import re
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from django.conf import settings
from django.utils.encoding import force_str
from honeypot.decorators import verify_honeypot_value

# these were moved out of Django 1.2 -- we're going to still use them
_POST_FORM_RE = re.compile(
    r'(<form\W[^>]*\bmethod\s*=\s*(\'|"|)POST(\'|"|)\b[^>]*>)', re.IGNORECASE
)
_HTML_TYPES = ("text/html", "application/xhtml+xml")


class BaseHoneypotMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)


class HoneypotViewMiddleware(BaseHoneypotMiddleware):
    """
    Middleware that verifies a valid honeypot on all non-ajax POSTs.
    """

    def process_view(self, request, callback, callback_args, callback_kwargs):
        # replace old is_ajax method that was removed in Django 3.1
        if request.META.get("HTTP_X_REQUESTED_WITH") == "XMLHttpRequest":
            return None
        if getattr(callback, "honeypot_exempt", False):
            return None
        return verify_honeypot_value(request, None)


class HoneypotResponseMiddleware(BaseHoneypotMiddleware):
    """
    Middleware that rewrites all POST forms to include honeypot field.
    """

    def __call__(self, request):
        response = self.get_response(request)

        try:
            content_type = response["Content-Type"].split(";")[0]
        except (KeyError, AttributeError):
            content_type = None

        if content_type in _HTML_TYPES:
            # ensure we don't add the 'id' attribute twice (HTML validity)
            def add_honeypot_field(match):
                """Returns the matched <form> tag plus the added <input> element"""
                value = getattr(settings, "HONEYPOT_VALUE", "")
                if callable(value):
                    value = value()
                return mark_safe(
                    match.group()
                    + render_to_string(
                        "honeypot/honeypot_field.html",
                        {"fieldname": settings.HONEYPOT_FIELD_NAME, "value": value},
                    )
                )

            # Modify any POST forms
            response.content = _POST_FORM_RE.sub(
                add_honeypot_field, force_str(response.content)
            )
        return response


class HoneypotMiddleware(HoneypotViewMiddleware, HoneypotResponseMiddleware):
    """
    Combines HoneypotViewMiddleware and HoneypotResponseMiddleware.
    """



from functools import wraps
from django.conf import settings
from django.http import HttpResponseBadRequest
from django.template.loader import render_to_string


def honeypot_equals(val):
    """
    Default verifier used if HONEYPOT_VERIFIER is not specified.
    Ensures val == HONEYPOT_VALUE or HONEYPOT_VALUE() if it's a callable.
    """
    expected = getattr(settings, "HONEYPOT_VALUE", "")
    if callable(expected):
        expected = expected()
    return val == expected


def verify_honeypot_value(request, field_name):
    """
    Verify that request.POST[field_name] is a valid honeypot.

    Ensures that the field exists and passes verification according to
    HONEYPOT_VERIFIER.
    """
    verifier = getattr(settings, "HONEYPOT_VERIFIER", honeypot_equals)
    if request.method == "POST":
        field = field_name or settings.HONEYPOT_FIELD_NAME
        if field not in request.POST or not verifier(request.POST[field]):
            resp = render_to_string(
                "honeypot/honeypot_error.html", {"fieldname": field}
            )
            return HttpResponseBadRequest(resp)


def check_honeypot(func=None, field_name=None):
    """
    Check request.POST for valid honeypot field.

    Takes an optional field_name that defaults to HONEYPOT_FIELD_NAME if
    not specified.
    """
    # hack to reverse arguments if called with str param
    if isinstance(func, str):
        func, field_name = field_name, func

    def decorated(func):
        def inner(request, *args, **kwargs):
            response = verify_honeypot_value(request, field_name)
            if response:
                return response
            else:
                return func(request, *args, **kwargs)

        return wraps(func)(inner)

    if func is None:

        def decorator(func):
            return decorated(func)

        return decorator
    return decorated(func)


def honeypot_exempt(view_func):
    """
    Mark view as exempt from honeypot validation
    """
    # borrowing liberally from django's csrf_exempt
    def wrapped(*args, **kwargs):
        return view_func(*args, **kwargs)

    wrapped.honeypot_exempt = True
    return wraps(view_func)(wrapped)



from django import template
from django.conf import settings

register = template.Library()


@register.inclusion_tag("honeypot/honeypot_field.html")
def render_honeypot_field(field_name=None):
    """
    Renders honeypot field named field_name (defaults to HONEYPOT_FIELD_NAME).
    """
    if not field_name:
        field_name = settings.HONEYPOT_FIELD_NAME
    value = getattr(settings, "HONEYPOT_VALUE", "")
    if callable(value):
        value = value()
    return {"fieldname": field_name, "value": value}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['honeypot', 'honeypot.templatetags']

package_data = \
{'': ['*'], 'honeypot': ['templates/honeypot/*']}

install_requires = \
['Django>=2.2,<5.0']

setup_kwargs = {
    'name': 'django-honeypot',
    'version': '1.0.3',
    'description': 'Django honeypot field utilities',
    'long_description': '===============\ndjango-honeypot\n===============\n\n.. image:: https://github.com/jamesturk/django-honeypot/workflows/Test/badge.svg\n\n.. image:: https://img.shields.io/pypi/v/django-honeypot.svg\n    :target: https://pypi.python.org/pypi/django-honeypot\n\nDjango application that provides utilities for preventing automated form spam.\n\nProvides template tags, view decorators, and middleware to add and verify honeypot fields to forms.\n\nWritten by James Turk with contributions by Flavio Curella and Daniel Greenfeld.\n\nSource: https://github.com/jamesturk/django-honeypot/\n\nRequirements\n============\n\n* python >= 3.7\n* django >= 2.2\n\n(django-honeypot 0.7 supports Django 1.11 and Python 2.7)\n\nUsage\n=====\n\nsettings.py\n-----------\n\nBe sure to add ``honeypot`` to ``INSTALLED_APPS`` in settings.py.\n\nYou will almost always need to define ``HONEYPOT_FIELD_NAME`` which is the name to use for the honeypot field.  Some sophisticated bots will attempt to avoid fields named honeypot, so it may be wise to name the field something slightly more realistic such as "phonenumber" or "body2".\n\n``HONEYPOT_VALUE`` is an option that you can specify to populate the honeypot field, by default the honeypot field will be empty and any text entered into it will result in a failed POST.  ``HONEYPOT_VALUE`` can be a string or a callable that takes no arguments.\n\n``HONEYPOT_VERIFIER`` is an advanced option that you can specify to validate the honeypot.  The default verifier ensures that the contents of the honeypot field matches ``HONEYPOT_VALUE``.  Using a combination of a callable for ``HONEYPOT_VALUE`` and ``HONEYPOT_VERIFIER`` it is possible to implement a more advanced technique such as using timestamps.\n\nAdding honeypot fields to specific forms and views\n--------------------------------------------------\n\nIt is possible to add honeypot fields to specific forms and ensure that specific views check for a valid honeypotin ``request.POST``.  This can be accomplished by using the ``render_honeypot_field`` template tag:\n\nAt the top of a template file include the line::\n\n    {% load honeypot %}\n\nAnd then within any form including the tag::\n\n    {% render_honeypot_field "field_name" %}\n\nwill render a honeypot field named "field_name" that is hidden by default.  The name of the honeypot field will default to ``HONEYPOT_FIELD_NAME`` if one is not provided.\n\nTo ensure that the honeypot field is both present and correct you will need to use ``check_honeypot`` decorator from ``honeypot.decorators``:\n\n.. code:: python\n\n    from honeypot.decorators import check_honeypot\n\n    @check_honeypot(field_name=\'hp_field_name\')\n    def post_comment(request):\n        ...\n\n    @check_honeypot\n    def other_post_view(request):\n        ...\n\nThis decorator will ensure that a field exists in ``request.POST`` that is named \'field_name\'.  ``@check_honeypot`` without arguments will use the default ``HONEYPOT_FIELD_NAME``.\n\nAdding honeypot fields site-wide\n--------------------------------\n\nSometimes it is desirable to add honeypots to all forms site-wide.  This is particularly useful when dealing with apps that render their own forms.  For this purpose three middlewares are provided, similar in functionality to django\'s own CSRF middleware.\n\nAll of these middleware live in ``honeypot.middleware``.\n\n``HoneypotResponseMiddleware`` analyzes the output of all responses and rewrites any forms that use ``method="POST"`` to contain a honeypot field, just as if they had started with ``{% render_honeypot_field %}``.  Borrowing heavily from ``django.contrib.csrf.middleware.CsrfResponseMiddleware`` this middleware only rewrites responses with Content-Type text/html or application/xhtml+xml.\n\n``HoneypotViewMiddleware`` ensures that for all incoming POST requests to views ``request.POST`` contains a valid honeypot field as defined by the ``HONEYPOT_FIELD_NAME``, ``HONEYPOT_VALUE``, and ``HONEYPOT_VERIFIER`` settings.  The result is the same as if every view in your project were decorated with ``@check_honeypot``.\n\n``HoneypotMiddleware`` is a combined middleware that applies both ``HoneypotResponseMiddleware`` and ``HoneypotViewMiddleware``, this is the easiest way to get honeypot fields site-wide and can be used in many if not most cases.\n\nCustomizing honeypot display\n----------------------------\n\nThere are two templates used by django-honeypot that can be used to control various aspects of how the honeypot functionality is presented to the user.\n\n``honeypot/honeypot_field.html`` is used to render the honeypot field.  It is given two context variables ``fieldname`` and ``value``, corresponding to ``HONEYPOT_FIELD_NAME`` and ``HONEYPOT_VALUE`` or any overrides in effect (such as a custom field name passed to the template tag).\n\n``honeypot/honeypot_error.html`` is the error page rendered when a bad request is intercepted.  It is given the context variable ``fieldname`` representing the name of the honeypot field.\n\n',
    'author': 'James Turk',
    'author_email': 'dev@jamesturk.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jamesturk/django-honeypot/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
